module State where

import Data.Array
import System.Random

type Pos = (Int, Int)

-- Position is valid (for this state)
validPos :: State-> Pos-> Bool
validPos st (x, y) = 0 <= x && x <= xMax st && 0 <= y && y <= yMax st

-- The game state
data State = St { fields :: Array Pos Bool
                , self   :: Pos
                , moves  :: Int
                }

-- Maximum x position
xMax :: State-> Int
xMax = fst. snd. bounds. fields

-- Maximum y position
yMax :: State-> Int
yMax = snd. snd. bounds. fields

-- Increment the move counter
incMov :: State-> State
incMov st = st {moves = moves st+ 1}

-- Create an initial state
initialState :: Int-> Int-> IO State
initialState x y = do
  rndx <- mapM randomRIO (replicate (x*y `div` 30) (0, x))
  rndy <- mapM randomRIO (replicate (x*y `div` 30) (0, y))     
  let mt   = array ((0, 0), (x, y)) [((i, j), False) | i<- [0.. x], j<- [0.. y]]
      flds = mt // [((x, y), True) | (x, y) <- zip rndx rndy]
      self = (x `div` 2, y `div` 2)
  return $ St flds self 0



-- Rendering the state
renderField :: State-> Pos-> Char
renderField st p 
  | fields st ! p = '^'
  | p == self st = '@' 
  | otherwise = ' '

render :: State-> String
render st =
 let line i = '|': [renderField st (x, i) | x<- [0.. xMax st]] ++ "|\n"
     brdr   = '+' : replicate (xMax st+ 1) '-' ++ "+\n"
     stln   = "Move: "++ show (moves st) 
 in  brdr ++ concatMap line [0.. yMax st] ++ brdr ++ stln
